@@ -10,7 +10,7 @@ from lintian_brush.control import (
     ensure_minimum_version,
     get_relation,
     read_debian_compat_file,
-    update_control,
+    ControlUpdater,
     )
 from lintian_brush.debhelper import (
     detect_debhelper_buildsystem,
@@ -90,26 +90,23 @@ if os.path.exists('debian/compat'):
 
     current_debhelper_compat_version = read_debian_compat_file('debian/compat')
     if current_debhelper_compat_version < new_debhelper_compat_version:
-        with open('debian/compat', 'w') as f:
-            f.write('%s\n' % new_debhelper_compat_version)
+        with open('debian/compat', 'w') as cf:
+            cf.write('%s\n' % new_debhelper_compat_version)
     else:
         # Nothing to do
         sys.exit(2)
 
-    def bump_debhelper(control):
-        control["Build-Depends"] = ensure_minimum_version(
-                control.get("Build-Depends", ""),
-                "debhelper",
-                "%d~" % new_debhelper_compat_version)
-
-    update_control(source_package_cb=bump_debhelper)
+    with ControlUpdater() as updater:
+        updater.source["Build-Depends"] = ensure_minimum_version(
+            updater.source.get("Build-Depends", ""),
+            "debhelper",
+            "%d~" % new_debhelper_compat_version)
 else:
     # Assume that the compat version is set in Build-Depends
-    def bump_debhelper_compat(control):
-        global current_debhelper_compat_version
+    with ControlUpdater() as updater:
         try:
             offset, debhelper_compat_relation = get_relation(
-                control.get("Build-Depends", ""), "debhelper-compat")
+                updater.source.get("Build-Depends", ""), "debhelper-compat")
         except KeyError:
             sys.exit(2)
         else:
@@ -122,13 +119,11 @@ else:
             current_debhelper_compat_version = int(
                 debhelper_compat_relation[0].version[1])
         if current_debhelper_compat_version < new_debhelper_compat_version:
-            control["Build-Depends"] = ensure_exact_version(
-                    control["Build-Depends"],
+            updater.source["Build-Depends"] = ensure_exact_version(
+                    updater.source["Build-Depends"],
                     "debhelper-compat",
                     "%d" % new_debhelper_compat_version)
 
-    update_control(source_package_cb=bump_debhelper_compat)
-
 
 # For a list of behavior changes between debhelper compat verisons, see
 # https://manpages.debian.org/testing/debhelper/debhelper.7.en.html#Supported_compatibility_levels
@@ -356,6 +351,21 @@ def rename_installsystemd_target(rule):
         b'override_dh_systemd_start', b'override_dh_installsystemd')
 
 
+def upgrade_to_debhelper_10():
+
+    # dh_installinit will no longer install a file named debian/package as an
+    # init script.
+
+    with ControlUpdater() as updater:
+        for binary in updater.binaries:
+            name = binary['Package']
+            if os.path.isfile(os.path.join('debian', name)):
+                os.rename(os.path.join('debian', name),
+                          os.path.join('debian', name + '.init'))
+                subitems.add(
+                    'Rename debian/%s to debian/%s.init.' % (name, name))
+
+
 def upgrade_to_debhelper_11():
 
     update_rules(
@@ -376,9 +386,31 @@ def upgrade_to_debhelper_11():
                 service, str(current_package_version())))
 
 
+def upgrade_to_debhelper_13():
+
+    # dh_installtempfiles will handle d/tmpfile{,s}.  It prefer the latter
+    # and warns about the former.
+
+    with ControlUpdater() as updater:
+        for binary in updater.binaries:
+            name = binary['Package']
+            tmpfile = os.path.join('debian', name + ".tmpfile")
+            if os.path.isfile(tmpfile):
+                os.rename(tmpfile, tmpfile + 's')
+                subitems.add(
+                    'Rename %s to %ss.' % (tmpfile, tmpfile))
+        if os.path.isfile(os.path.join('debian', "tmpfile")):
+            tmpfile = os.path.join('debian', "tmpfile")
+            os.rename(tmpfile, tmpfile + 's')
+            subitems.add(
+                'Rename %s to %ss.' % (tmpfile, tmpfile))
+
+
 upgrade_to_debhelper = {
+    10: upgrade_to_debhelper_10,
     11: upgrade_to_debhelper_11,
     12: upgrade_to_debhelper_12,
+    13: upgrade_to_debhelper_13,
 }
 
 
