@@ -30,7 +30,7 @@ locale.setlocale(locale.LC_ALL, '')
 # Use better default than ascii with posix filesystems that deal in bytes
 # natively even when the C locale or no locale at all is given. Note that
 # we need an immortal string for the hack, hence the lack of a hyphen.
-sys._brz_default_fs_enc = "utf8"
+sys._brz_default_fs_enc = "utf8"   # type: ignore
 
 import breezy  # noqa: E402
 from breezy.errors import (
@@ -198,9 +198,6 @@ def main(argv=None):
                 minimum_certainty = cfg.minimum_certainty()
             if compat_release is None:
                 compat_release = cfg.compat_release()
-                if compat_release:
-                    compat_release = debian_info.codename(
-                        compat_release, default=compat_release)
             if allow_reformatting is None:
                 allow_reformatting = cfg.allow_reformatting()
             if update_changelog is None:
