@@ -37,6 +37,9 @@ from debian.changelog import Changelog
 from . import reset_tree
 
 
+DEFAULT_DEBIAN_PATCHES_DIR = 'debian/patches'
+
+
 def find_patch_base(tree):
     """Find the base revision to apply patches to.
 
@@ -170,7 +173,7 @@ def read_quilt_series(f):
         yield patch, quoted, options
 
 
-def read_quilt_patches(tree, directory='debian/patches'):
+def read_quilt_patches(tree, directory=DEFAULT_DEBIAN_PATCHES_DIR):
     """Read patch contents from quilt directory.
 
     Args:
@@ -219,16 +222,75 @@ def upstream_with_applied_patches(tree, patches):
         yield tree
 
 
-def tree_non_patches_changes(tree):
-    """Check if a Debian tree has changes vs upstream tree.
+def rules_find_patches_directory(makefile):
+    """Find the patches directory set in debian/rules.
+
+    Args:
+        makefile: Makefile to scan
+    Returns:
+        path to patches directory, or None if none was found in debian/rules
+    """
+    try:
+        val = makefile.get_variable(b'QUILT_PATCH_DIR')
+    except KeyError:
+        return None
+    else:
+        return val.decode()
+
+
+def find_patches_directory(path):
+    """Find the name of the patches directory, if any.
+
+    Args:
+      path: Root to package
+    Returns:
+      relative path to patches directory, or None if none exists
+    """
+    from .rules import Makefile
+    directory = None
+    try:
+        mf = Makefile.from_path(os.path.join(path, 'debian/rules'))
+    except FileNotFoundError:
+        pass
+    else:
+        rules_directory = rules_find_patches_directory(mf)
+        if rules_directory is not None:
+            directory = rules_directory
+    if directory is None and os.path.exists(
+            os.path.join(path, DEFAULT_DEBIAN_PATCHES_DIR)):
+        directory = DEFAULT_DEBIAN_PATCHES_DIR
+    return directory
+
+
+def tree_patches_directory(tree):
+    """Find the name of the patches directory.
+
+    This will always return a path, even if the patches
+    directory does not yet exist.
 
     Args:
       tree: Tree to check
+    Returns:
+      path to patches directory, or what it should be
+    """
+    directory = find_patches_directory(tree.abspath('.'))
+    if directory is None:
+        return DEFAULT_DEBIAN_PATCHES_DIR
+    return directory
+
+
+def tree_non_patches_changes(tree, patches_directory):
+    """Check if a Debian tree has changes vs upstream tree.
+
+    Args:
+      tree: Tree to check
+      patches_directory: Name of patches directory
     Returns:
         list of TreeDelta objects
     """
-    directory = 'debian/patches'
-    patches = list(read_quilt_patches(tree, directory))
+    if patches_directory is None:
+        return
+    patches = list(read_quilt_patches(tree, patches_directory))
 
     # TODO(jelmer): What if patches are already applied on tree?
     with upstream_with_applied_patches(tree, patches) \
@@ -295,24 +357,26 @@ def find_common_patch_suffix(names, default='.patch'):
     return max(suffix_count.items(), key=lambda v: v[1])[0]
 
 
-def add_patch(name, contents, header=None):
+def add_patch(tree, patches_directory, name, contents, header=None):
     """Add a new patch.
 
     Args:
+      tree: Tree to edit
+      patches_directory: Name of patches directory
       name: Patch name without suffix
       contents: Diff
       header: RFC822 to read
     Returns:
       Name of the patch that was written (including suffix)
     """
-    if not os.path.isdir('debian/patches'):
-        os.mkdir('debian/patches')
-    patch_suffix = find_common_patch_suffix(os.listdir('debian/patches'))
+    if not tree.has_filename(patches_directory):
+        tree.mkdir(patches_directory)
+    patch_suffix = find_common_patch_suffix(os.listdir(patches_directory))
     patchname = name + patch_suffix
-    path = os.path.join('debian/patches', patchname)
-    if os.path.exists(path):
+    path = os.path.join(patches_directory, patchname)
+    if tree.has_filename(path):
         raise FileExistsError(path)
-    with open(path, 'wb') as f:
+    with open(tree.abspath(path), 'wb') as f:
         if header is not None:
             f.write(header.as_string().encode('utf-8'))
             f.write(b'\n')
@@ -320,7 +384,8 @@ def add_patch(name, contents, header=None):
 
     # TODO(jelmer): Write to patches branch if applicable
 
-    with open('debian/patches/series', 'a') as f:
+    series_path = os.path.join(patches_directory, 'series')
+    with open(tree.abspath(series_path, 'a')) as f:
         f.write('%s\n' % patchname)
 
     return patchname
@@ -346,10 +411,12 @@ def move_upstream_changes_to_patch(
     lines = description.splitlines()
     header['Description'] = lines[0].rstrip('\n')
     header.set_payload(''.join([l + '\n' for l in lines[1:]]).lstrip())
-    patchname = add_patch(patch_name, diff.getvalue(), header)
+    patches_directory = tree_patches_directory(local_tree)
+    patchname = add_patch(
+        local_tree, patches_directory, patch_name, diff.getvalue(), header)
     specific_files = [
-        os.path.join(subpath, 'debian/patches'),
-        os.path.join(subpath, 'debian/patches/series'),
-        os.path.join(subpath, 'debian/patches', patchname)]
+        os.path.join(subpath, patches_directory),
+        os.path.join(subpath, patches_directory, 'series'),
+        os.path.join(subpath, patches_directory, patchname)]
     local_tree.add(specific_files)
     return specific_files, patchname
