@@ -318,17 +318,22 @@ def guess_from_debian_watch(path, trust_package):
             return
         for w in wf:
             url = w.format_url(package=get_package_name)
+            if 'mode=git' in w.options:
+                yield UpstreamDatum(
+                    "Repository", sanitize_vcs_url(url), "confident")
+                continue
             if url.startswith('https://') or url.startswith('http://'):
                 repo = guess_repo_from_url(url)
                 if repo:
                     yield UpstreamDatum(
                         "Repository", sanitize_vcs_url(repo), "likely")
-                    break
+                    continue
             m = re.match('https?://sf.net/([^/]+)', url)
             if m:
                 yield UpstreamDatum("Archive", "SourceForge", "certain")
                 yield UpstreamDatum(
                     "X-SourceForge-Project", m.group(1), "certain")
+                continue
 
 
 def guess_from_debian_control(path, trust_package):
@@ -542,20 +547,26 @@ def guess_from_readme(path, trust_package):
                     yield UpstreamDatum(
                         'Bug-Database',
                         m.group(0).decode().rstrip(), 'possible')
-                m = re.fullmatch(b'https://github.com/([^/]+)/([^/?.]+)', line)
+                for m in re.finditer(
+                        b'https://github.com/([^/]+)/([^/\\s()]+)(.git)?',
+                        line):
+                    yield UpstreamDatum(
+                        'Repository', m.group(0).decode().rstrip(), 'possible')
+                m = re.fullmatch(
+                    b'https://github.com/([^/]+)/([^/?.()]+)', line)
                 if m:
                     yield UpstreamDatum(
-                        'Repository', line.strip().decode(), 'likely')
+                        'Repository', line.strip().decode(), 'possible')
                 m = re.fullmatch(b'git://([^ ]+)', line)
                 if m:
                     yield UpstreamDatum(
-                        'Repository', line.strip().decode(), 'likely')
+                        'Repository', line.strip().decode(), 'possible')
                 for m in re.finditer(
-                        b'https://([^/]+)/([^ ]+)', line):
+                        b'https://([^/]+)/([^\\s()]+)', line):
                     if is_gitlab_site(m.group(1).decode()):
                         yield UpstreamDatum(
                             'Repository',
-                            m.group(0).decode().rstrip(), 'likely')
+                            m.group(0).decode().rstrip(), 'possible')
     except IsADirectoryError:
         pass
 
@@ -1258,6 +1269,11 @@ def verify_bug_database_url(url):
         except urllib.error.HTTPError as e:
             if e.status == 404:
                 return False
+            if e.status == 403:
+                # Probably rate limited
+                warn('Unable to verify bug database URL %s: %s' % (
+                     url, e.reason))
+                return None
             raise
         return data['has_issues'] and not data.get('archived', False)
     if is_gitlab_site(parsed_url.netloc):
@@ -1679,8 +1695,13 @@ def guess_from_launchpad(package, distribution=None, suite=None):
         distribution = 'ubuntu'
     if suite is None:
         if distribution == 'ubuntu':
-            from distro_info import UbuntuDistroInfo
-            suite = UbuntuDistroInfo().devel()
+            from distro_info import UbuntuDistroInfo, DistroDataOutdated
+            ubuntu = UbuntuDistroInfo()
+            try:
+                suite = ubuntu.devel()
+            except DistroDataOutdated as e:
+                warn(str(e))
+                suite = ubuntu.all[-1]
         elif distribution == 'debian':
             suite = 'sid'
     sourcepackage_url = (
